/*  ESP32_WebServer_twoled
 *   
 *  Egyszerű webszerver mintapélda, amelyben két LED-et kapcsolhatunk
 *  a web böngészőből (LED1 a GPIO22, LED2 a GPIO23 lábakra kötve).
 *  
 *  A program eredeti változata a Last Minute Engineers honlapon 
 *  található. két változatban (AP és STA módban)
 *  Create A Simple ESP32 Web Server In Arduino IDE
 *  Link: https://lastminuteengineers.com/creating-esp32-web-server-arduino-ide/
 *
 *  Mi az 'ESP32 as HTTP Server using WiFi Station (STA) mode'
 *  című második változatot vettük át, kissé módosítva:
 *   - A LED-eket más GPIO lábakra kötöttük
 *   - A nyitólapon nem kapcsoljuk le a LED-eket, mint az eredetiben
 *   - A hibejelzéshez HTML lapot készítettünk egy nyomógombbal, 
 *     amely megnyomáskor a nyitóoldalra visz vissza
 *     
 * Tested with Arduini IDE 1.8.19 and ESP32 Arduino Core 2.0.2    
 */

#include <WiFi.h>
#include <WebServer.h>
#include "secrets.h"
WebServer server(80);      

uint8_t LED1pin = 22;
bool LED1status = LOW;

uint8_t LED2pin = 23;
bool LED2status = LOW;

void setup() {
  Serial.begin(115200);
  delay(100);
  pinMode(LED1pin, OUTPUT);
  pinMode(LED2pin, OUTPUT);
  setup_wifi();

  server.on("/", handle_OnConnect);
  server.on("/led1on", handle_led1on);
  server.on("/led1off", handle_led1off);
  server.on("/led2on", handle_led2on);
  server.on("/led2off", handle_led2off);
  server.onNotFound(handle_NotFound);
  server.begin();
  Serial.println("HTTP server started");
}

void loop() {
  server.handleClient();
  if (LED1status)
  { digitalWrite(LED1pin, HIGH); }
  else
  { digitalWrite(LED1pin, LOW);  }

  if (LED2status)
  { digitalWrite(LED2pin, HIGH); }
  else
  { digitalWrite(LED2pin, LOW);  }
}

void handle_OnConnect() {
  Serial.println("GPIO22 Status: OFF | GPIO23 Status: OFF");
  server.send(200, "text/html", SendHTML(LED1status, LED2status));
}

void handle_led1on() {
  LED1status = HIGH;
  Serial.println("GPIO22 Status: ON");
  server.send(200, "text/html", SendHTML(true, LED2status));
}

void handle_led1off() {
  LED1status = LOW;
  Serial.println("GPIO22 Status: OFF");
  server.send(200, "text/html", SendHTML(false, LED2status));
}

void handle_led2on() {
  LED2status = HIGH;
  Serial.println("GPIO23 Status: ON");
  server.send(200, "text/html", SendHTML(LED1status, true));
}

void handle_led2off() {
  LED2status = LOW;
  Serial.println("GPIO23 Status: OFF");
  server.send(200, "text/html", SendHTML(LED1status, false));
}

void handle_NotFound() {
  server.send(404, "text/html", SendErrorPage());
}

String SendHTML(uint8_t led1stat, uint8_t led2stat) {
  String ptr;
  ptr.reserve(1024);
  ptr = F("<!DOCTYPE html> <html>\n"
          "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n"
          "<title>LED Control</title>\n"
          "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n"
          "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #444444;margin-bottom: 50px;}\n"
          ".button {display: block;width: 80px;background-color: #3498db;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n"
          ".button-on {background-color: #3498db;}\n"
          ".button-on:active {background-color: #2980b9;}\n"
          ".button-off {background-color: #34495e;}\n"
          ".button-off:active {background-color: #2c3e50;}\n"
          "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n"
          "</style>\n"
          "</head>\n"
          "<body>\n"
          "<h1>ESP32 Web Server</h1>\n"
          "<h3>Using Station(STA) Mode</h3>\n");
  if (led1stat)
  { ptr += "<p>LED1 Status: ON</p><a class=\"button button-off\" href=\"/led1off\">OFF</a>\n"; }
  else
  { ptr += "<p>LED1 Status: OFF</p><a class=\"button button-on\" href=\"/led1on\">ON</a>\n"; }
  if (led2stat)
  { ptr += "<p>LED2 Status: ON</p><a class=\"button button-off\" href=\"/led2off\">OFF</a>\n"; }
  else
  { ptr += "<p>LED2 Status: OFF</p><a class=\"button button-on\" href=\"/led2on\">ON</a>\n"; }
  ptr += F("</body></html>\n");
  return ptr;
}

String SendErrorPage() {
  String ptr;
  ptr.reserve(1024);
  ptr = F("<!DOCTYPE html> <html>\n"
          "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n"
          "<title>LED Control</title>\n"
          "<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n"
          "body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #444444;margin-bottom: 50px;}\n"
          ".button {display: block;width: 100px;background-color: #3498db;border: none;color: white;padding: 13px 20px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n"
          "p {font-size: 14px;color: #888;margin-bottom: 10px;}\n"
          "</style>\n"
          "</head>\n"
          "<body>\n"
          "<h1>404 Page not found</h1>\n"
          "<h3>Press button to open start page</h3>\n" 
          "<a class=\"button\" href=\"/\">RESTART</a>\n"          
          "</body></html>\n");
  return ptr;
}

void setup_wifi() {
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  Serial.print("Connecting to ");
  Serial.print(WIFI_SSID);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.print("Connected! IP address: ");
  Serial.println(WiFi.localIP());
}
